import deepMerge from 'deepmerge';
export function mergeResolvers(resolversDefinitions) {
    if (!resolversDefinitions || resolversDefinitions.length === 0) {
        return {};
    }
    if (resolversDefinitions.length === 1) {
        return resolversDefinitions[0];
    }
    const resolversFactories = new Array();
    const resolvers = new Array();
    for (const resolversDefinition of resolversDefinitions) {
        if (typeof resolversDefinition === 'function') {
            resolversFactories.push(resolversDefinition);
        }
        else if (typeof resolversDefinition === 'object') {
            resolvers.push(resolversDefinition);
        }
    }
    if (resolversFactories.length) {
        return ((...args) => {
            const resultsOfFactories = resolversFactories.map(factory => factory(...args));
            return deepMerge.all([...resolvers, ...resultsOfFactories]);
        });
    }
    else {
        return deepMerge.all(resolvers);
    }
}
//# sourceMappingURL=merge-resolvers.js.map