// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
import { GraphQLInputObjectType, GraphQLList, GraphQLNonNull } from 'graphql';
import { DESIGN_TYPE } from './common';
import { getScalarTypeFromClass } from '.';
const GRAPHQL_INPUT_OBJECT_TYPE_CONFIG = 'graphql:input-object-type-config';
const GRAPHQL_INPUT_TYPE = 'graphql:input-type';
export function InputField(typeFactory, config) {
    return (target, propertyKey) => {
        const existingConfig = Reflect.getMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, target.constructor) || {};
        const inputFieldName = (config && config.name) || propertyKey;
        const inputFieldType = typeFactory ? typeFactory() : Reflect.getMetadata(DESIGN_TYPE, target, propertyKey);
        const inputFieldGraphQLType = getInputTypeFromClass(inputFieldType) || getScalarTypeFromClass(inputFieldType) || inputFieldType;
        const inputFieldConfig = {
            type: (config && 'nullable' in config && !config.nullable) ? new GraphQLNonNull(inputFieldGraphQLType) : inputFieldGraphQLType,
        };
        existingConfig.fields = existingConfig.fields || {};
        existingConfig.fields[inputFieldName] = Object.assign({}, (existingConfig.fields[inputFieldName] || {}), inputFieldConfig);
        Reflect.defineMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
    };
}
export function InputObjectType(config = {}) {
    return target => {
        const existingConfig = Reflect.getMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, target) || {};
        Reflect.defineMetadata(GRAPHQL_INPUT_TYPE, new GraphQLInputObjectType(Object.assign({ name: target.name }, existingConfig, config)), target);
        return target;
    };
}
export function getInputTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getInputTypeFromClass(target[0]);
        return elementType && new GraphQLList(elementType);
    }
    return Reflect.getMetadata(GRAPHQL_INPUT_TYPE, target);
}
//# sourceMappingURL=input-object-type.js.map