"use strict";
// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const common_1 = require("./common");
const _1 = require(".");
const GRAPHQL_INPUT_OBJECT_TYPE_CONFIG = 'graphql:input-object-type-config';
const GRAPHQL_INPUT_TYPE = 'graphql:input-type';
function InputField(typeFactory, config) {
    return (target, propertyKey) => {
        const existingConfig = Reflect.getMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, target.constructor) || {};
        const inputFieldName = (config && config.name) || propertyKey;
        const inputFieldType = typeFactory ? typeFactory() : Reflect.getMetadata(common_1.DESIGN_TYPE, target, propertyKey);
        const inputFieldGraphQLType = getInputTypeFromClass(inputFieldType) || _1.getScalarTypeFromClass(inputFieldType) || inputFieldType;
        const inputFieldConfig = {
            type: (config && 'nullable' in config && !config.nullable) ? new graphql_1.GraphQLNonNull(inputFieldGraphQLType) : inputFieldGraphQLType,
        };
        existingConfig.fields = existingConfig.fields || {};
        existingConfig.fields[inputFieldName] = Object.assign({}, (existingConfig.fields[inputFieldName] || {}), inputFieldConfig);
        Reflect.defineMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
    };
}
exports.InputField = InputField;
function InputObjectType(config = {}) {
    return target => {
        const existingConfig = Reflect.getMetadata(GRAPHQL_INPUT_OBJECT_TYPE_CONFIG, target) || {};
        Reflect.defineMetadata(GRAPHQL_INPUT_TYPE, new graphql_1.GraphQLInputObjectType(Object.assign({ name: target.name }, existingConfig, config)), target);
        return target;
    };
}
exports.InputObjectType = InputObjectType;
function getInputTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getInputTypeFromClass(target[0]);
        return elementType && new graphql_1.GraphQLList(elementType);
    }
    return Reflect.getMetadata(GRAPHQL_INPUT_TYPE, target);
}
exports.getInputTypeFromClass = getInputTypeFromClass;
//# sourceMappingURL=input-object-type.js.map